%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This script is to creat the synthetic data from images.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear everything
clear all;
close all;
clc;

% The data folder.
path  = '/Users/quoctd/Data/MyPostDoc/Epfl-Svn/matlab/poisson_image/data/';

% Define the blurring matrix.
blur = 1e-4*[ ...
   0.011178978813925, 0.050100707167907, 0.082602101584847, 0.050100707167907, 0.011178978813925; ...
   0.050100707167907, 0.224535791730617, 0.370196935859762, 0.224535791730617, 0.050100707167907; ...
   0.082602101584847, 0.370196935859762, 0.610351562500000, 0.370196935859762, 0.082602101584847; ...
   0.050100707167907, 0.224535791730617, 0.370196935859762, 0.224535791730617, 0.050100707167907; ...
   0.011178978813925, 0.050100707167907, 0.082602101584847, 0.050100707167907, 0.011178978813925];

% Define the operator A (blurring).
A  = @(x)( conv2(x,blur,'same') );

% This operator is used for denoising.
%A  = @(x)( 1e-4*x );

% The list of images
imlist = {'aircraft', 'barbara', 'boat', 'cameraman', 'einstein', ...
          'epfl', 'lena', 'hanoi', 'house', 'peppers', 'barbara256', ...
          'boat256', 'cameraman256', 'car' };
      
imlist = { 'cameraman256'};

% Scaling factor.
scale_factor = 10; 

% Generate all the images.
for index = 1:length(imlist)
    
    % Image name
    iname = imlist{index};

    % Load the image
    im    = im2uint16(imread([path, iname, '.png']));
    im    = round(double(im(:,:,1)));
    f     = im(:);
    
    % Scale the image by a given scale factor;
    f     = scale_factor*f;
    
    % Apply the poisson distribution on the image.
    fstar = A(f);
    y     = poissrnd(fstar);
    y     = round(y);

    % Reshape data.
    M     = size(im, 1);
    N     = size(im, 2);
    y     = reshape(y, M, N);
    f     = reshape(f, M, N);

    % Save the data.
    %save([path, iname, num2str(scale_factor), '.mat'], 'y', 'f', 'blur');
    save([path, iname, '.mat'], 'y', 'f', 'blur');

    % Size of the figure
    fprintf('Size = (%d, %d)\n', M, N);

    % Plot images.
    figure(1);
    title('Noise image');
    imagesc(reshape(y, M, N)); colormap gray; axis square;
    figure(2);
    title('Original image');
    imagesc(reshape(im, M, N)); colormap gray; axis square;
    figure(3);
    title('Scaled image');
    imagesc(reshape(f, M, N)); colormap gray; axis square;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% End of the implementation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%