function op = proj_prox_l1( q )

%PROX_L1    L1 norm.
%    OP = PROX_L1( q ) implements the nonsmooth function
%        OP(X) = norm(q.*X,1).
%    Q is optional; if omitted, Q=1 is assumed. But if Q is supplied,
%    then it must be a positive real scalar (or must be same size as X).
% Dual: proj_linf.m

% Update Feb 2011, allowing q to be a vector
% Update Mar 2012, allow stepsize to be a vector

if nargin == 0,
	q = 1;
elseif ~isnumeric( q ) || ~isreal( q ) ||  any( q < 0 ) || all(q==0) %|| numel( q ) ~= 1
	error( 'Argument must be positive.' );
end

op = tfocs_prox( @f, @prox_f , 'vector' ); % Allow vector stepsizes

function v = f(x)
    x1 = x(:);
    x1 = x(1:end-1);
    v = norm( q(:).*x1(:), 1 );
end

function xout = prox_f(x, t)
    x1    = x(1:end-1);
    s1    = x(end);
    tq    = t .* q; % March 2012, allowing vectorized stepsizes
    s     = 1 - min( tq./abs(x1), 1 );
    xout  = x1.* s;
    xout  = xout(:);
    xout  = [xout; max(s1, 0)];
end


end


% TFOCS v1.2 by Stephen Becker, Emmanuel Candes, and Michael Grant.
% Copyright 2012 California Institute of Technology and CVX Research.
% See the file TFOCS/license.{txt,pdf} for full license information.

