% PURPOSE: Test the scopt_glearn_ls_pna solver with some synthetic data.
%
%

%***************************Synthetic data ******************************
% Input data
n   = 100;       % Problem dimension.
rho = 1e-2;      % Regularize parameter.

% Problem dimension.
fprintf('+ The size of problem: %d\n', n);
fprintf('+ The regularization parameter: %6.5f\n', rho);
fprintf('+ Generating the data ...\n');
N         = round(5*n);          % number of samples.
percent   = 0.03;
SIGMA     = 0.219224;

% generate a sparse positive definite inverse covariance matrix.
Sinv      = diag(abs(ones(n,1)));
idx       = randsample(n^2, round(percent*n^2));
Sinv(idx) = ones(numel(idx), 1);
Sinv      = Sinv + Sinv';        % make symmetric.
if min(eig(Sinv)) < 0            % make positive definite.
    Sinv  = Sinv + 1.1*abs(min(eig(Sinv)))*eye(n);
end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
S = inv(Sinv);                   % true covariance matrix.

% Generate Gaussian samples.
D = mvnrnd(zeros(1,n), S, N);

% Sample covariance matrix.
S_hat = cov(D);
fprintf('Rank of sample covariance matrix S_hat: %d\n', rank(S_hat));

% %***************************** Real data ********************************
% % Input data.
% rho   = 0.6;          % Regularization parameter.
% pname = 'Lymph.mat';  % Data file.
% 
% % Load the data.
% dataDir = which('scopt_glearn_pga_exam.m');
% dataDir = [dataDir(1:end-23), 'glearn_data/'];
% if exist([dataDir, pname])
%     data  = load([dataDir, pname]);
%     S_hat = data.S;
%     p     = size(S_hat, 1);
% else
%     error('Cannot load the input data');
% end

% Default parameters.
param = scopt_optimset([]);

% User defined parameters.
param.AbsTol      = 1e-7;   % Absolute tolerance.
param.RelTol      = 1e-7;   % Relative tolerance.
param.MaxIters    = 200;    % Maximum number of iterations.
param.SubMaxIters = 500;    % Maximum number of iterations in subsolver.
param.SubRelTol   = 1e-10;  % Tolerance for terminating subsolver.
param.isFxEval    = true;   % Require to evaluate the objective value.
param.numFxCheck  = 4;      % Terminate if the objective value does not 
                            % change significantly after 4 successive
                            % iterations.
param.isTrace     = 0;      % Do not keep trace.
param.BtkMaxIters = 200;    % Maximum number of line-search iterations.
param.PrintLevel  = 2;      % Print 50 iterations for each line.
param.QuadCvRg    = 0;

% Generate an initial point.
X0    = diag(1./diag(S_hat));

%------------------------------------------------------------------
% Call the algorithm without line-search using TFOCS for subsolver.
%------------------------------------------------------------------
param.SubMethod = 'BCDS';
param.isBtkLS   = false;
fprintf('+ CALL THE SOLVER (without line-search) USING TFOCS ...\n');
[final_sol_nls, output_nls] = scopt_glearn_ls_pna(S_hat, rho, param, ...
                              X0, [], @scopt_tfocs_uLASSO);
% The final solution.
Xsol     = final_sol_nls.x;
fval_nls = -sum(log(eig(Xsol))) + trace(S_hat*Xsol) + rho*norm(Xsol(:), 1);
fprintf(' + The objective value (without line-search): %6.6f\n\n', fval_nls);

%---------------------------------------------------------------
% Call the algorithm with line-search using TFOCS for subsolver.
%---------------------------------------------------------------
if exist('scopt_tfocs_uLASSO.m')
    % Choose the method for solving the subproblem: User-defined solver
    % (TFOCS).
    param.SubMethod   = 'user-defined';
    %param.isBtkLS = true;
    fprintf('+ CALL THE SOLVER (with line-search) USING TFOCS ...\n');
    [final_sol_ls, output_ls] = scopt_glearn_ls_pna(S_hat, rho, param, X0, ...
                                [], @scopt_tfocs_uLASSO);

    % Evaluate the objective function.
    Xsol    = final_sol_ls.x;
    fval_ls = -sum(log(eig(Xsol))) + trace(S_hat*Xsol) + rho*norm(Xsol(:), 1);
    fprintf(' + The objective value (with line-search): %6.6f\n\n', fval_ls);
end

%-------------------------------------------------------------------
% Call the algorithm with line-search using L1General for subsolver.
%-------------------------------------------------------------------
if exist('scopt_l1general_uLASSO.m')
    % Choose the method for solving the subproblem: User-defined solver
    % (L1-General).
    param.SubMethod   = 'user-defined';
    %param.isBtkLS = true;
    fprintf('+ CALL THE SOLVER (with line-search) USING L1GENERAL ...\n');
    [final_sol_l1, output_l1] = scopt_glearn_ls_pna(S_hat, rho, param, X0, ...
                                [], @scopt_l1general_uLASSO);

    % Evaluate the objective function.
    Xsol    = final_sol_l1.x;
    fval_l1 = -sum(log(eig(Xsol))) + trace(S_hat*Xsol) + rho*norm(Xsol(:), 1);
    fprintf(' + The objective value (with line-search): %6.6f\n\n', fval_l1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%