% PURPOSE: Using L1General to solve the unconstrained LASSO problem
% DATE: 01.07.2013
% BY    Quoc Tran Dinh, LIONS, EPFL, Lausanne, Switzerland.
%

function [xsol, fval, output] = scopt_l1general_uLASSO(Aopr, b, rho, ...
                                ATopr, x0, param)
              

% Define the smooth term.
smoothF = @(x)(smooth_fx(x, Aopr, ATopr, b));

% Reset the options.
opts.order   = 2;   
opts.maxIter = param.MaxIters;
opts.optTol  = param.RelTolX;
opts.verbose = 0;

% Define vector of regularization parameter.
rho_v = rho*ones(size(x0));

% Call L1General.
%[xsol, out] = L1General2_PSSgb(smoothF, x0, rho_v, opts);
%[xsol, out] = L1General2_BBST(smoothF, x0, rho_v, opts);
%[xsol, out] = L1General2_AS(smoothF, x0, rho_v, opts);
[xsol, out] = L1General2_OPG(smoothF, x0, rho_v, opts);

% Assign the outputs.
fval            = [];
output.iter     = out.iter;
output.msg      = '';
output.error    = 0;
if isfield(out, 'prox_cnt')
    output.prox_cnt = out.prox_cnt;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Function: Define the smooth term.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fx, dfx] = smooth_fx(x, Aopr, ATopr, b)

Axb = Aopr(x) - b;
fx  = 0.5*Axb'*Axb;
dfx = ATopr(Axb);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

