% PURPOSE: Using TFOCS to solve the unconstrained LASSO problem
% DATE: 01.07.2013
% BY    Quoc Tran Dinh, LIONS, EPFL, Lausanne, Switzerland.
%

function [xsol, fval, output] = scopt_tfocs_uLASSO(Aopr, b, rho, ...
                                ATopr, x0, param)
              

% Define the smooth term.
smoothF = @(x)(smooth_fx(x, Aopr, ATopr, b));

% Reset the options.
opts.maxIts     = param.SubMaxIters;
opts.tol        = param.SubRelTol;
opts.printEvery = 0;
opts.alg        = 'N07';
opts.restart    = 100;

% Call TFOCS.
[xsol, out, out_opts] = tfocs(smoothF, [], prox_l1(rho), x0, opts);

% Assign the outputs.
fval            = out.f(end);
output.iter     = out.niter;
output.msg      = out.status;
output.error    = out.normGrad(end);
if isfield(out, 'prox_cnt')
    output.prox_cnt = out.prox_cnt;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Function: Define the smooth term.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fx, dfx] = smooth_fx(x, Aopr, ATopr, b)

Axb = Aopr(x) - b;
fx  = 0.5*Axb'*Axb;
dfx = ATopr(Axb);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

