function [xsol, sig, output] = tfocs_exam(X, y, rho, param)

% Define the smooth term of the objective value.
fx    = @(x)(smooth_fx(x, X, y));

% Set options.
opts.maxIts       = param.MaxIters;
opts.tol          = param.RelTolX;
opts.alg          = param.Method;
opts.restart      = param.Restart;

% Generate an initial point.
x0 = [zeros(size(X,2), 1); exp(1)]; 

% Call the TFOCS solver.
output.time  = tic;
[xsol_tfocs, output_tfocs, info] = tfocs(fx, [], proj_prox_l1(rho), x0, opts);
output.time  = toc(output.time);
output.tfocs = output_tfocs;
output.info  = info;
wsol         = xsol_tfocs;

sig     = 1/wsol(end);
xsol    = sig*wsol(1:end-1);

function [fx, dfx] = smooth_fx(x, X, y)

beta = x(1:end-1);
sig  = x(end);
z    = X*beta - sig*y;
n    = size(X,1);

fx   = -log(sig) + (0.5/n)*z'*z;
dfx  = [X'*z/n; -1/sig - z'*y/n];


