%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%        -------------------------------------------------------- 
%%%        SPARSE INVERSE COVARIANCE ESTIMATION IN GRAPHICAL MODELS
%%%                          Version 1.0.0 (2013)  
%%%        ---------------------------0*0*0------------------------
%%%
%%%    An implementation of the (Path-Following) Proximal Gradient/Newton 
%%%       algorithms for solving Sparse Inverse Covariance Estimation 
%%%                 Problems in Graphical Model Selection.
%%%
%%% INFORMATION:
%%%    By A. Kyrillidis, Laboratory for Informations and Inference Systems,
%%%       EPFL, Lausanne, Switzerland.
%%%    Joint work with Quoc Tran Dinh and Volkan Cevher.
%%%    Date: 26.11.2012.
%%%    Last modified: 27.05.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%% REFERENCES:
%%%    [1]. Q. Tran Dinh, A. Kyrillidis and V. Cevher: A proximal Newton 
%%%    framework for composite minimization: Graph learning without 
%%%    Cholesky decompositions and matrix inversions. ICML 2013, Alanta,
%%%    USA.
%%%
%%% FUNCTION: [DX, X0, lambda, output] = scopt_glearn_blcp_check_feas(...
%%%                                    d, beta, lambda, H, F_plus, F_minus) 
%%% PURPOSE:
%%%    Check the feasibility of the prediction point.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function infeasible = scopt_glearn_blcp_feas_check(d, beta, lambda,...
                                                   H, F_plus, F_minus)                                          
infeasible = 1;
z = norm(d(H), 'inf');
if (z <= lambda && sum(beta(F_plus) < 0) == 0 && sum(beta(F_minus) > 0) == 0) 
    infeasible = 0; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
