%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%        -------------------------------------------------------- 
%%%        SPARSE INVERSE COVARIANCE ESTIMATION IN GRAPHICAL MODELS
%%%                          Version 1.0.0 (2013)  
%%%        ---------------------------0*0*0------------------------
%%%
%%%    An implementation of the (Path-Following) Proximal Gradient/Newton 
%%%       algorithms for solving Sparse Inverse Covariance Estimation 
%%%                 Problems in Graphical Model Selection.
%%%
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 26.11.2012.
%%%    Last modified: 27.05.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%% REFERENCES:
%%%    [1]. Q. Tran Dinh, A. Kyrillidis and V. Cevher: A proximal Newton 
%%%    framework for composite minimization: Graph learning without 
%%%    Cholesky decompositions and matrix inversions. ICML 2013, Alanta,
%%%    USA.
%%%
%%%
%%% FUNCTION: fval = scopt_glearn_dqp_eval(Y, X_cur, Qt, mode)
%%% PURPOSE:  Evaluate the objective value of the quadratic function:
%%%                fval = 0.5*Y*H^{-1}*Y + Qt*Y.
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fval, gradf] = scopt_glearn_dqp_eval(Y, X_cur, Qt, mode)

% Check if we just want to compute the gradient.
if nargin < 4, mode = 0; end

% Compute the term XYX.
XY  = X_cur*Y;

% Evaluate the objective value/gradient.
% Evaluate the gradient vector.
if mode == true 
    XYX  = XY*X_cur;
    fval = XYX + Qt;
   
% Evaluate the objective value.    
else 
    fval = 0.5*sum(sum(XY'.*XY)) + sum(sum(Qt'.*Y));
end

% Evaluate the gradient vector.
if nargout > 1
    XYX   = XY*X_cur;
    gradf = XYX + Qt;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%