%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%        -------------------------------------------------------- 
%%%        SPARSE INVERSE COVARIANCE ESTIMATION IN GRAPHICAL MODELS
%%%                          Version 1.0.0 (2013)  
%%%        ---------------------------0*0*0------------------------
%%%
%%%    An implementation of the (Path-Following) Proximal Gradient/Newton 
%%%       algorithms for solving Sparse Inverse Covariance Estimation 
%%%                 Problems in Graphical Model Selection.
%%%
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 26.11.2012.
%%%    Last modified: 27.05.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%% REFERENCES:
%%%    [1]. Q. Tran Dinh, A. Kyrillidis and V. Cevher: A proximal Newton 
%%%    framework for composite minimization: Graph learning without 
%%%    Cholesky decompositions and matrix inversions. ICML 2013, Alanta,
%%%    USA.
%%%
%%%
%%% FUNCTION: [fval, Lx] = scopt_glearn_fx_eval(X, S_hat, rho, L)
%%% PURPOSE:  Evaluate the objective value of the function.
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fval, Lx] = scopt_glearn_fx_eval(X, S_hat, rho, L)

if nargin < 4,
    [L_tmp, cholfailed]   = chol(X);
    if cholfailed ~= 0
        fval = inf; 
        if nargout > 1, Lx = []; end
        return; 
    end
elseif isempty(L)
    [L_tmp, cholfailed]   = chol(X);
    if cholfailed ~= 0
        fval = inf; 
        if nargout > 1, Lx = []; end
        return; 
    end
else
    L_tmp = L;
end
fval = -2*sum(log(diag(L_tmp)));

fval = fval + sum(sum(S_hat'.*X));
fval = fval + rho*sum(sum(abs(X))); 

if nargout > 1, Lx = L_tmp; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%