%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%        -------------------------------------------------------- 
%%%        SPARSE INVERSE COVARIANCE ESTIMATION IN GRAPHICAL MODELS
%%%                          Version 1.0.0 (2013)  
%%%        ---------------------------0*0*0------------------------
%%%
%%%    An implementation of the (Path-Following) Proximal Gradient/Newton 
%%%       algorithms for solving Sparse Inverse Covariance Estimation 
%%%                 Problems in Graphical Model Selection.
%%%
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 26.11.2012.
%%%    Last modified: 27.05.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%% REFERENCES:
%%%    [1]. Q. Tran Dinh, A. Kyrillidis and V. Cevher: A proximal Newton 
%%%    framework for composite minimization: Graph learning without 
%%%    Cholesky decompositions and matrix inversions. ICML 2013, Alanta,
%%%    USA.
%%%
%%%
%%% FUNCTION: fval = scopt_glearn_qp_eval(X, X_cur, W_cur, Shat, mode)
%%% PURPOSE:  Evaluate the objective value of the quadratic function:
%%%                fval = 0.5*(x - x_cur)'*H*(x - x_cur) + h'*(x - x_cur).
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fval, gradf] = scopt_glearn_qp_eval(X, X_cur, W_cur, Shat, mode)

% Check if we just want to compute the gradient.
if nargin < 5, mode = 0; end

% The direction.
DX  = X - X_cur;

% The gradient vector
h   = Shat - W_cur;
WDX = W_cur*DX;

% Evaluate the objective value or gradient.
if mode == 1 % Evalute the gradient.
    fval = WDX*W_cur + h;
else % Evaluate the objective value.
    fval = 0.5*sum(sum(WDX'.*WDX)) + sum(sum(h.'*DX));
end

% Evaluate the gradient vector.
if nargout > 1
    gradf = WDX*W_cur + h;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%