%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: [fx, gradfx, qx_out, px_out] = scopt_logistic_fx_eval(x, ...
%%%                                        Ahat, bhat, N, NAThat, isFxEval) 
%%% PURPOSE:  Evaluate the objective value/gradient of the function.
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fx, gradfx, qx_out, px_out] = scopt_logistic_fx_eval(x, Ahat, ...
                                        bhat, N, NAThat, isFxEval) 
% Check the mode.
if nargin < 6,      isFxEval = true;  end
if nargin < 5,      NAThat = Ahat'/N; end
if isempty(NAThat), NAThat = Ahat'/N; end

% Evaluate the vector pxk = 1./(1 + exp(-A*x)) - 1.
Ax    = Ahat*x + bhat;
expAx = exp(-Ax);

% Evaluate the objective value.
if isFxEval == true
    fx = (1/N)*sum( log(1 + expAx) );
else
    fx = inf;
end

% Evaluate the gradient.
if nargout > 1
    px     = 1./(1 + expAx) - 1;
    qx     = expAx./(1 + expAx).^2;
    gradfx = NAThat*px;
    if nargout > 2, qx_out = qx; end
    if nargout > 3, px_out = px; end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%