%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: [best_fx, indxx] = scopt_logistic_plot(output, file_name, ...
%%%                                                  issemilog)
%%% PURPOSE: Ploting the final output.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [best_fx, indxx] = scopt_logistic_plot(output, file_name, issemilog)

if nargin < 3, issemilog = true; end

fx_val = [];
if isfield(output, 'fval_pg'),  fx_val = [fx_val; output.fval_pg];  end
if isfield(output, 'fval_fpg'), fx_val = [fx_val; output.fval_fpg]; end
if isfield(output, 'fval_fpg1'), fx_val = [fx_val; output.fval_fpg1]; end
if isfield(output, 'fval_fpg2'), fx_val = [fx_val; output.fval_fpg2]; end
if isfield(output, 'fval_pn1'), fx_val = [fx_val; output.fval_pn1]; end
if isfield(output, 'fval_pn2'), fx_val = [fx_val; output.fval_pn2]; end
if isfield(output, 'fval_pn3'), fx_val = [fx_val; output.fval_pn3]; end
if isfield(output, 'fval3'),    fx_val = [fx_val; output.fval3];    end
if isfield(output, 'fval4'),    fx_val = [fx_val; output.fval4];    end
if isfield(output, 'fval5'),    fx_val = [fx_val; output.fval5];    end
if isfield(output, 'fval6'),    fx_val = [fx_val; output.fval6];    end
if isfield(output, 'fval7'),    fx_val = [fx_val; output.fval7];    end
if isfield(output, 'fval8'),    fx_val = [fx_val; output.fval8];    end

if isempty(fx_val)
    disp('No data to plot!!!');
    return; 
end

factor  = 0.0;
[best_fx, indxx] = min(fx_val);
best_fx = best_fx - factor*abs(best_fx);
namestr = '';

% Plot the objective values.
figure('Name', ['Objective values for problem: ', file_name]);

% Normal gradient methods.
if isfield(output, 'pginfo')
    y1 = (output.pginfo.info.fx - best_fx)/best_fx; 
    if issemilog, semilogy(y1, 'b--'); else plot(y1, 'b--'); end
    hold on;
    if isempty(namestr)
        namestr = {'Normal PG'};
    else
        namestr = {namestr{:}, 'Normal PG'};
    end
end

% Fast gradient method.
if isfield(output, 'fpginfo')
    y1f = (output.fpginfo.info.fx - best_fx)/best_fx;
    if issemilog, semilogy(y1f, 'r-.'); else plot(y1f, 'r-.'); end
    hold on;
    if isempty(namestr)
        namestr = {'Fast PG'};
    else
        namestr = {namestr{:}, 'Fast PG'};
    end
end

% Fast gradient method with restart.
if isfield(output, 'fpginfo1')
    y2f = (output.fpginfo1.info.fx - best_fx)/best_fx;
    if issemilog, semilogy(y2f, 'c--'); else plot(y2f, 'c--'); end
    hold on;
    if isempty(namestr)
        namestr = {'Fast PG (RS)'};
    else
        namestr = {namestr{:}, 'Fast PG (RS)'};
    end
end

% Fast gradient method with restart and backtracking.
if isfield(output, 'fpginfo2')
    y3f = (output.fpginfo2.info.fx - best_fx)/best_fx;
    if issemilog, semilogy(y3f, 'k--'); else plot(y3f, 'k--'); end
    hold on;
    if isempty(namestr)
        namestr = {'Fast PG (RS-BT)'};
    else
        namestr = {namestr{:}, 'Fast PG(RS-BT)'};
    end
end

% Proximal-Newton method.
if isfield(output, 'pninfo1')
    y2 = (output.pninfo1.info.fx - best_fx)/best_fx;
    if issemilog, semilogy(y2, 'r-'); else plot(y2, 'r-'); end
    hold on;
    if isempty(namestr)
        namestr = {'PN (exact)'};
    else
        namestr = {namestr{:}, 'PN (exact)'};
    end
end

% Proximal-Newton method with BFGS.
if isfield(output, 'pninfo2')
    y3 = (output.pninfo2.info.fx - best_fx)/best_fx;
    if issemilog, semilogy(y3, 'g-.'); else plot(y3, 'g-.'); end
    hold on;
    if isempty(namestr)
        namestr = {'PN (BFGS)'};
    else
        namestr = {namestr{:}, 'PN (BFGS)'};
    end
end

% Proximal-Newton method with LBFGS.
if isfield(output, 'pninfo3')
    y4 = (output.pninfo3.info.fx - best_fx)/best_fx;
    if issemilog, semilogy(y4, 'k-.'); else plot(y4, 'k-.'); end
    hold on;
    if isempty(namestr)
        namestr = {'PN (LBFGS)'};
    else
        namestr = {namestr{:}, 'PN (LBFGS)'};
    end
end

% TFOCS solver.
if isfield(output, 'tfocs')
    y5 = (output.tfocs.infor1.f - best_fx)/best_fx; 
    if issemilog, semilogy(y5, 'm--'); else plot(y5, 'm--'); end
    hold on;
    if isempty(namestr)
        namestr = {'TFOCS (AT)'};
    else
        namestr = {namestr{:}, 'TFOCS (AT)'};
    end
end

% TFOCS solver with restart.
if isfield(output, 'tfocs2')
    y51 = (output.tfocs2.infor1.f - best_fx)/best_fx; 
    if issemilog, semilogy(y51, 'm-'); else plot(y51, 'm-'); end
    hold on;
    if isempty(namestr)
        namestr = {'TFOCS (AT-RS)'};
    else
        namestr = {namestr{:}, 'TFOCS (AT-RS)'};
    end
end

% TFOCS solver with N07.
if isfield(output, 'tfocs3')
    y52 = (output.tfocs3.infor1.f - best_fx)/best_fx; 
    if issemilog, semilogy(y52, 'k-'); else plot(y52, 'm-'); end
    hold on;
    if isempty(namestr)
        namestr = {'TFOCS (N07)'};
    else
        namestr = {namestr{:}, 'TFOCS (N07)'};
    end
end

% PNOPT with BFGS update.
if isfield(output, 'pnopt1')
    y6 = (output.pnopt1.Trace.f_x - best_fx)/best_fx;
    if issemilog, semilogy(y6, 'k-'); else plot(y6, 'k-'); end
    if isempty(namestr)
        namestr = {'PNOPT-Bfgs'};
    else
        namestr = {namestr{:}, 'PNOPT-Bfgs'};
    end
end

% PNOPT with Limitted-BFGS update.
if isfield(output, 'pnopt2')
    y7 = (output.pnopt2.Trace.f_x - best_fx)/best_fx;
    if issemilog, semilogy(y7, 'm-.'); else plot(y7, 'm-.'); end
    if isempty(namestr)
        namestr = {'PNOPT-Lbfgs'};
    else
        namestr = {namestr{:}, 'PNOPT-Lbfgs'};
    end
end

% Display the legend.
if ~isempty(namestr)
    legend(namestr);
    hold on;
end
shg;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%