%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: [FX, DFX, PX] = scopt_mnlogistic_fx_eval(X, What, Yhat, ...
%%%                                                    mWhat, TN, mode)
%%% PURPOSE: Evaluate the objective value and its gradient vector.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FX, DFX, PX] = scopt_mnlogistic_fx_eval(X, What, Yhat, TN, ...
                         mWhat, mode)
    
% Check the inputs.
if nargin < 6,    mode = 'FX'; end
if isempty(mode), mode = 'FX'; end

% The size of samples.
N     = size(What, 2);
p     = size(What, 1);
mx    = size(Yhat, 1);

% Generate the temporary data.
one1N  = ones(1, N);
onep1  = ones(p, 1);
if nargin < 5
    mWhat   = repmat(What, mx, 1);
end
if isempty(mWhat)
    mWhat   = repmat(What, mx, 1);
end

% If X is a vector, we need to reshape it.
if isvector(X)
    X = reshape(X, p, mx);
end

% Evaluate Phat - the probability matrix.
XTWhat   = X'*What;                   % XTWhat is a matrix of size mxN.
Zhat     = exp(XTWhat);               % Zhat is also a matrix of size mxN.
sumZhat2 = sum([one1N; Zhat], 1);     % sumZhat2 is a vector of size 1xN.

% Evaluate the objective value at X, if required.
if strcmpi(mode, 'DFX')
    Phat   = Zhat./repmat(sumZhat2, mx, 1); % This is mxN matrix.
    PYhat  = Phat - Yhat;
    FX     = sum( kron(PYhat, onep1).*mWhat, 2);
    if isvector(FX)
        FX = reshape(FX, p, mx);
    end
    FX     = FX/TN;
else
    FX     = sum(log(sumZhat2)) - sum(sum(Yhat.*XTWhat));
    FX     = FX/TN;
end

% Evaluate the gradient of FX.
if nargout > 1
    
    % Evaluate the gradient of Fhat.
    Phat    = Zhat./repmat(sumZhat2, mx, 1); % This is mxN matrix.
    PYhat   = Phat - Yhat;
    DFX     = sum( kron(PYhat, onep1).*mWhat, 2);
    if isvector(DFX)
        DFX = reshape(DFX, p, mx);
    end
    DFX     = DFX/TN;
    if nargout > 2
        PX  = Phat;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
