%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%                ----------------------------------------- 
%%%                POISON IMAGE RECONSTRUCTION v.1.0. (2013)
%%%                ----------------------------------------- 
%%%
%%%    An implementation of the (Path-Following) Proximal Gradient/Newton 
%%%    algorithms for solving Poisson image reconstruction problems.
%%%
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 22.05.2013.
%%%    Last modified: 22.05.2013.
%%%    Contact: quoc.trandinh@epfl.ch, volkan.cevher@epfl.ch
%%% 
%%% FUNCTION: Fval = scopt_poisson_fx_eval(x, Aopr, yf, tau, beta, varargin)
%%% PURPOSE:  Evaluate the objective value, i.e.:
%%%           Fval = Af - sum(y_i*log(Af + beta)) + tau*||f||_TV.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Fval = scopt_poisson_fx_eval(x, Aopr, yf, tau, beta, varargin)

% Check if Aopr is an operator.
if isa(Aopr, 'function_handle')
    Ax = Aopr(x);
else
    Ax = Aopr;
end

Fval = sum(sum(Ax)) - sum(sum(yf.*log(Ax + beta)));
if tau > 0 && ~isinf(tau) && ~isempty(tau)
    if nargin > 5
        Fval = Fval + tau.*scopt_poisson_TVnorm(x, varargin{1});
    else
        Fval = Fval + tau.*scopt_poisson_TVnorm(x, 'l1');
    end
end

if nargin > 6
    Mf   = varargin{2};
    Fval = Mf*Fval;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

