%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%%                ----------------------------------------- 
%%%                POISON IMAGE RECONSTRUCTION v.1.0. (2013)
%%%                ----------------------------------------- 
%%%
%%%    An implementation of the (Path-Following) Proximal Gradient/Newton 
%%%    algorithms for solving Poisson image reconstruction problems.
%%%
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 22.05.2013.
%%%    Last modified: 22.05.2013.
%%%    Contact: quoc.trandinh@epfl.ch, volkan.cevher@epfl.ch
%%% 
%%% FUNCTION: l2norm = scopt_poisson_l2norm(X, Ax, yf, beta, Aopr, ...
%%%                                         ATopr, Mf, maxiter)
%%% PURPOSE: Evaluate the Lipschitz constant by a simple power method.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l2norm = scopt_poisson_l2norm(X, Ax, yf, beta, Aopr, ATopr, ...
                                       Mf, maxiter)
% Check the input parameters.
if nargin < 7, maxiter = 20; end

yfAxbeta = yf./(Ax + beta).^2;

% Power iterations.
dir      = ones(size(X));
for iter = 1:maxiter
    Ad   = Aopr(dir);
    Df   = yfAxbeta.*Ad;
    Df   = Mf*ATopr(Df);
    dir  = Df / norm(Df(:), 2);
end

% Compute L2-norm.
Ad     = Aopr(dir);
dHd    = yfAxbeta.*(Ad.^2);
l2norm = Mf*sum(sum(dHd)) / sum(sum(dir.*dir));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%