% PURPOSE: Setup SCOPT
% Date: 01.07.2013
% By Quoc Tran Dinh, LIONS, EPFL, Lausanne, Switzerland
%

% MATLAB path
if ~exist('scopt_setup.m')
    error( ['The SCOPT solver has not been installed yet! ', ...
            'Download at http://lions.epfl.ch/software/SCOPT.'] );
end
mpath      = matlabpath;
scopt_path = which( 'scopt_setup.m' );

% Find the root folder of the scp solver
if ispc
    fs = '\'; ps = ';';  exts = ';';
else
    fs = '/'; ps = ':';  exts = ':';
end
tmp    = strfind( scopt_path, fs );
scopt_path(tmp(end):length(scopt_path)) = [];
subdir = { 'utilities', 'examples', 'functions', 'doc', ...
           'examples/glearn_data', 'examples/logistic_data', ...
           'examples/mnlogistic_data', 'examples/poisson_data' };

% Update the folders fot the scp solver
strpath = '';
for k=1:length( subdir );
    tst = subdir{k};
    tid = ( tst == '/');
    if any( tid ), tst(tid) = fs; end
    tmp = strcat( scopt_path, fs, tst );
    if exist( tmp, 'dir' )
        tmp2 = strcat( tmp, exts );
        ndxs = strfind( mpath, tmp2 );
        if isempty( ndxs )
            strpath = strcat( strpath, tmp, ';');
        end
    else
        error( 'The folder does not exist!' ); 
    end
end   

% Set back to the matlab path
if ~isempty( strpath )
    mpath = strcat( strpath, mpath );
    matlabpath( mpath );  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%