%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: H = scopt_bfgs_update(H_old, s, y)
%%% PURPOSE:  Update matrix H by the BFGS formula. Here, we use Powell's 
%%%           trick to avoid the nonnegativity of s'*y.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function H = scopt_bfgs_update(H_old, s, y)

% Make sure that s and y are vectors.
s = s(:);
y = y(:);

% Check the size.
if length(s) ~= length(y) || length(s) ~= size(H_old, 1) ...
   || length(s) ~= size(H_old, 2)
    error('The size is not consistent!');
end

% Compute sTHs and sy.
sy   = sum(s.*y);
Hs   = H_old*s;
sTHs = sum(Hs.*s);

% Generate vector r.
r = y;
if sy < 0.2*sTHs
    theta = 0.8*sTHs/(sTHs - sy);
    r     = theta*y + (1-theta)*Hs;
end

% Update the BFGS formula.
H = H_old - (Hs*Hs')/(sTHs) + (r*r')/(r'*s);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%