%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: BBstep = compute_BB_step(x_old, x, gradf_old, gradf, param)
%%% PURPOSE:  Compute the Barzilai-Borwein step size.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function BBstep = scopt_compute_BBstep(x_old, x, gradf_old, gradf, param, L0)

% Return the default value.
if nargin < 6, L0 = 0.05; end
if param.BBStepType == 0, BBstep = L0; return; end
if isempty(x_old) || isempty(gradf_old), BBstep = L0; return; end

% Evaluate the Barzilai-Borwein step.
y_cur  = gradf - gradf_old;
s_cur  = x     - x_old;
s_cur  = s_cur(:); % Make sure that y and s are vectors.
y_cur  = y_cur(:);
sumSx2 = sum(s_cur.*s_cur);
BBstep = L0;
if sumSx2 > 0
    if param.BBStepType == 2
        BBstep = sum(sum(y_cur.*y_cur))/sumSx2;
    elseif param.BBStepType == 3
        BBstep = sum(sum(y_cur.*s_cur))/sumSx2;
    else
        BBstep = sqrt(sum(sum(y_cur.*y_cur))/sumSx2);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%