%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: scopt_copyright(param, varargin)
%%% PURPOSE:  Print the information of the solver.
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       (LIONS), EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 03.06.2013.
%%%    Last modified: 03.06.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function scopt_copyright(param, varargin)

if nargin < 1,     param = scopt_optimset([]); end
if isempty(param), param = scopt_optimset([]); end 
    
if param.PrintLevel <= 0, return; end
if nargin > 1, n_sp = varargin{1}; else n_sp = 72; end
fprintf('%s\n\n', repmat('*', 1, n_sp));
fprintf('')            
fprintf('             ---------------------------------------------\n');
fprintf('             SELF-CONCORDANT (LIKE) COMPOSITE MINIMIZATION \n');
fprintf('                              SCOPT 1.0.0\n');
fprintf('             ---------------------------------------------\n\n');
fprintf(' SCOPT is a MATLAB software package for solving composite\n'); 
fprintf(' minimization problems of a self-concordant (like) function and a\n'); 
fprintf(' possibly nonsmooth convex function. It consits of several algorithms\n');
fprintf(' including (but not restricted to) the proximal gradient, \n');
fprintf(' fast proximal gradient, proximal Newton and proximal quasi-Newton\n');
fprintf(' methods. Moreover, a path following proximal Newton method is\n');
fprintf(' also implemented for certain applications.\n');
fprintf(' For more information, we refer to our website: http://lion.epfl.ch.\n\n');
fprintf(' INFORMATION:\n');

fprintf('  By Quoc Tran Dinh\n');
fprintf('    Laboratory for Information and Inference Systems (LIONS)\n');
fprintf('    EPFL, Lausanne, Switzerland.\n');
fprintf('  Joint work with Volkan Cevher and Anastasios Kyrillidis (LIONS).\n');
fprintf('  Contact: quoc.trandinh@epfl.ch, volkan.cevher@epfl.ch\n');
fprintf('  Date: 03.06.2013.\n');
fprintf('  Last modified: 01.07.2013.\n\n');
if nargin < 2, return; end
    
% Print the method.
ss = 'Unknown';
if strcmpi(param.Algorithm, 'PG')
    ss = 'Proximal - Gradient Algorithm';
end
if strcmpi(param.Algorithm, 'FBPG')
    ss = 'Function-based Proximal - Gradient Algorithm';
end
if strcmpi(param.Algorithm, 'FPG')
    ss = 'Fast Proximal - Gradient Algorithm';
end
if strcmpi(param.Algorithm, 'PN')
    ss = 'Proximal - Newton Algorithm';
end
if strcmpi(param.Algorithm, 'PFPN')
    ss = 'Path-Following Proximal - Newton Algorithm';
end

% Check the detail option.
sss = '';
if strcmpi(param.SubMethod, 'dN07')
    sss = [sss, 'Nes07 dual-prox method for the dual subproblem'];
elseif strcmpi(param.SubMethod, 'pN07')
    sss = [sss, 'Nes07 dual-prox method for the primal subproblem'];
elseif strcmpi(param.SubMethod, 'pFISTA')
    sss = [sss, 'FISTA method for the primal subproblem'];
elseif strcmpi(param.SubMethod, 'dFISTA')
    sss = [sss, 'FISTA method for the dual subproblem'];
elseif strcmpi(param.SubMethod, 'dpFISTA')
    sss = [sss, 'FISTA method for primal and dual subproblems'];
elseif strcmpi(param.SubMethod, 'blcp')
    sss = [sss, 'Block active set'];
elseif strcmpi(param.SubMethod, 'pdAS')
    sss = [sss, 'Primal-dual active set'];
elseif strcmpi(param.SubMethod, 'pdAS2')
    sss = [sss, 'Primal-dual active set'];
elseif strcmpi(param.SubMethod, 'BCDS')
    sss = [sss, 'Block-coordinate descent'];
elseif strcmpi(param.SubMethod, 'user-defined')
    sss = [sss, 'User-defined LASSO method'];
end
if param.isBtkLS
    if strcmpi(param.lsMode, 'EBTK')
        sss = [sss, '-Enhanced Backtracking-']; 
    elseif strcmpi(param.lsMode, 'FWLK')
        sss = [sss, '-Forwardlooking-']; 
    else
        sss = [sss, '-Backtracking-']; 
    end
end
if param.isBtkLS
    sss = [sss, '-Line search-']; 
else
    sss = [sss, '-No-Line search-']; 
end
if strcmpi(param.isSubRestart, 'fixed') || strcmpi(param.isRestart, 'fixed')
    sss = [sss, '-Fixed Restart-']; 
elseif strcmpi(param.isSubRestart, 'fx') || strcmpi(param.isRestart, 'fx')
    sss = [sss, '-Adaptive Restart f(x)-']; 
end

ns1   = length(ss);
ns2   = length(sss);
ns0   = max(ns1, ns2);
nsp1  = round(max(0, (n_sp - ns1)/2 - 3));
nsp2  = round(max(0, (n_sp - ns2)/2 - 3));
nsp0  = min(nsp1, nsp2);
fprintf('%s%s\n', repmat(' ', 1, nsp0), repmat('*', 1, ns0 + 8));
fprintf('%s*** %s ***\n', repmat(' ', 1, nsp1), ss);
fprintf('%s*** %s ***\n', repmat(' ', 1, nsp2), sss);
fprintf('%s%s\n\n', repmat(' ', 1, nsp0), repmat('*', 1, ns0 + 8));
fprintf('%s\n', repmat('*', 1, n_sp));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%