%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: scopt_finaloutput(param, fout, varargin)
%%% PURPOSE:  Print the final output.
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       (LIONS), EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 03.06.2013.
%%%    Last modified: 03.06.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function scopt_finaloutput(param, fout, varargin)

% Print the final outputs.
if param.PrintLevel <= 0, return; end

if nargin > 2, n_sp = varargin{1}; else n_sp = 60; end
n_sp2 = round(max(n_sp/2 - 8, 0)); 
fprintf('%s', repmat('*', 1, n_sp2));
fprintf('THE FINAL OUTPUT');
fprintf('%s\n', repmat('*', 1, n_sp2));

if isfield(fout, 'msg')
    fprintf(' + Status: %s\n', fout.msg);
end
if isfield(fout, 'iter')
    fprintf(' + Number of total iterations: %6d\n', fout.iter);
end
if isfield(fout, 'iter1')
    fprintf(' + Number of iterations in Phase I: %6d\n', fout.iter1);
end
if isfield(fout, 'iter2')
    fprintf(' + Number of iterations in Phase II: %6d\n', fout.iter2);
end
if param.PrintLevel > 1
    if isfield(fout, 'iiter')
        fprintf(' + Number of inner iterations: %6d\n', fout.iiter);
    end
end
if isfield(fout, 'time')
    fprintf(' + CPU time (s): %6.5f\n', fout.time);
end
if isfield(fout, 'time1')
    fprintf(' + CPU time of Phase I (s): %6.5f\n', fout.time1);
end
if isfield(fout, 'time2')
    fprintf(' + CPU time of Phase II (s): %6.5f\n', fout.time2);
end
if param.isFxEval
    if isfield(fout, 'fval')
        fprintf(' + The final objective value: %6.12f\n', fout.fval);
    end
end
if param.TerminateType ~= 2 
    if isfield(fout, 'error')
        fprintf(' + The relative error: %6.12f\n', fout.error);
    end
end
if isfield(fout, 'prox_cnt')
    fprintf(' + The number of prox-operations: %6d\n', fout.prox_cnt);
end
fprintf('%s\n', repmat('*', 1, n_sp));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%