%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: [Fx, JFx] = scopt_gradfx_eval(func, X)
%%% PURPOSE:  Evaluate the gradient of the function fx.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Fx, JFx] = scopt_gradfx_eval(func, X, varargin)

% Make sure that X is the vector.
X = X(:);

% Check input variables.
step = sqrt(eps);

% Get dimension of vector x and create a indentity matrix E.
n  = length(X);
En = eye(n);

% Evaluate the function value at X.
Fx = feval(func, X, varargin{:});

% Evaluate the gradient vector by finite-difference method.
for k=1:n
    Fx_next   = feval( func, X + step*En(:, k ), varargin{:} );
    JFx(k, :) = (Fx_next - Fx)/step;
end

%%%% Here is also a way to compute gradient using i-trick.
% for k=1:n
%     
%     diff = feval( func, X + i*step*En( :, k ), varargin{:} );
%     JFx(k, :) = imag( diff/step );
%     
% end
% 
% Fx  = real(diff);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

