%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: l2nrm = scopt_l2normx(X, maxiter)
%%% PURPOSE:  Evaluate the maximum eigenvalue of matrix X by power method.
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       (LIONS), EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 03.06.2013.
%%%    Last modified: 03.06.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l2nrm = scopt_l2normx(X, maxiter)

if size(X, 1) ~= size(X, 2), error('Input matrix must be square!'); end
if nargin < 2, maxiter = 20; end

dir = ones(size(X,1),1);
for iter = 1:maxiter
    dir = (X*dir)/norm(dir, 2);
end
l2nrm = dir'*(X*dir)/(dir'*dir);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%