%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: [gradf, fx] = scopt_lbfgs_update(z, x, De, s, y, h, mode)
%%% PURPOSE:  Update matrix H by the limitted memory BFGS formula.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [gradf, fx] = scopt_lbfgs_update(z, x, De, s, y, h, mode)

ns   = size(s, 2);
L    = zeros(ns);
for k = 1:ns
    L(k+1:ns, k) = s(:, k+1:ns)'*y(:, k);
end
D1   = sum(s.*y);
D2   = sqrt(D1);

Q    = De*(s'*s) + L*(diag(1./D1)*L');
R    = chol(Q, 'lower');
R1   = [diag(D2), zeros(ns); -L*diag(1./D2), R];
R2   = [-diag(D2), diag(1./D2)*L'; zeros(ns), R'];
Qty2 = [y, De*s];

% Compute H*dx.
dx   = z - x;
Hdx  = (De*dx - Qty2*(R2\ (R1\(Qty2'*dx))));

% Compute the gradient vector.
gradf = [];
if mode == 1
    gradf = Hdx + h;
end

% Evaluate the function value.
if nargout > 1
    fx = sum((0.5*Hdx + h).*dx);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%