%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FUNCTION: info = scopt_trace(param, info, varargin)
%%% PURPOSE:  Trace the information along the iteration.
%%% INFORMATION:
%%%    By Quoc Tran Dinh, Laboratory for Informations and Inference Systems,
%%%       (LIONS), EPFL, Lausanne, Switzerland.
%%%    Joint work with Volkan Cevher and Anastasios Kyrillidis.
%%%    Date: 03.06.2013/
%%%    Last modified: 03.06.2013.
%%%    Contact: quoc.trandinh@epfl.ch
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function info = scopt_trace(param, info, varargin)

% Initialize the list.
if ~param.isTrace, return; end
if isempty(info)
    info.fx     = [];
    info.error  = [];
    info.prox   = [];
    info.step   = [];
    info.lambda = [];
    info.beta   = [];
    info.r      = [];
    info.decf   = [];
    info.param  = [];
    info.siter  = [];
    info.stime  = [];
    info.serr   = [];
    info.Lips   = [];
    info.x      = {};
end

% Collect the main information: f(x), error, prox.
if nargin > 2
    info.fx     = [info.fx;     varargin{1}];
    info.error  = [info.error;  varargin{2}];
    info.prox   = [info.prox;   varargin{3}];
end

% Collect orther information:
% step, lambda, beta, r, decf, param, siter, stime, Lips.
if param.TraceMode > 0
    if nargin > 5,  info.step   = [info.step;   varargin{4} ]; end
    if nargin > 6,  info.lambda = [info.lambda; varargin{5} ]; end
    if nargin > 7,  info.beta   = [info.beta;   varargin{6} ]; end
    if nargin > 8,  info.r      = [info.r;      varargin{7} ]; end
    if nargin > 9,  info.decf   = [info.decf;   varargin{8} ]; end
    if nargin > 10, info.param  = [info.param;  varargin{9} ]; end
    if nargin > 11, info.siter  = [info.siter;  varargin{10}]; end
    if nargin > 12, info.stime  = [info.stime;  varargin{11}]; end
    if nargin > 13, info.serr   = [info.serr;   varargin{12}]; end
    if nargin > 14, info.Lips   = [info.Lips;   varargin{13}]; end
end

% Collect the iterative vectors if required.
if param.TraceMode > 1
    if nargin > 15, info.x      = [info.x;    {varargin{14}}]; end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF THE IMPLEMENTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%